@@ -101,6 +101,22 @@ nm_meta_setting_info_editor_get_property_info (const NMMetaSettingInfoEditor *se
 	return NULL;
 }
 
+gboolean
+nm_meta_setting_info_editor_has_secrets (const NMMetaSettingInfoEditor *setting_info)
+{
+	guint i;
+
+	if (!setting_info)
+		return FALSE;
+
+	for (i = 0; i < setting_info->properties_num; i++) {
+		if (setting_info->properties[i]->is_secret)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 const NMMetaPropertyInfo *
 nm_meta_property_info_find_by_name (const char *setting_name, const char *property_name)
 {
@@ -257,6 +273,7 @@ nm_meta_abstract_info_complete (const NMMetaAbstractInfo *abstract_info,
                                 gpointer environment_user_data,
                                 const NMMetaOperationContext *operation_context,
                                 const char *text,
+                                gboolean *out_complete_filename,
                                 char ***out_to_free)
 {
 	const char *const*values;
@@ -276,6 +293,7 @@ nm_meta_abstract_info_complete (const NMMetaAbstractInfo *abstract_info,
 	                                                 environment_user_data,
 	                                                 operation_context,
 	                                                 text,
+	                                                 out_complete_filename,
 	                                                 out_to_free);
 
 	nm_assert (!*out_to_free || values == (const char *const*) *out_to_free);
