@@ -1,6 +1,4 @@
 #!/usr/bin/env python
-# vim: ft=python ts=4 sts=4 sw=4 et ai
-# -*- Mode: python; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,14 +19,16 @@
 
 # This example lists currently active connections
 
-from gi.repository import GLib, NMClient
+import gi
+gi.require_version('NM', '1.0')
+from gi.repository import GLib, NM
 
 if __name__ == "__main__":
-    client = NMClient.Client.new()
+    client = NM.Client.new(None)
     acons = client.get_active_connections()
     for ac in acons:
-        print "%s (%s) - %s" % (ac.get_id(), ac.get_uuid(), ac.get_connection_type())
+        print("%s (%s) - %s" % (ac.get_id(), ac.get_uuid(), ac.get_connection_type()))
     if len(acons) == 0:
-       print "No active connections"
+       print("No active connections")
 
 
