@@ -33,11 +33,11 @@
  * necessary for Open vSwitch interfaces.
  **/
 
-enum {
-	PROP_0,
+/*****************************************************************************/
+
+NM_GOBJECT_PROPERTIES_DEFINE_BASE (
 	PROP_TYPE,
-	LAST_PROP
-};
+);
 
 /**
  * NMSettingOvsInterface:
@@ -54,9 +54,7 @@ struct _NMSettingOvsInterfaceClass {
 	NMSettingClass parent;
 };
 
-G_DEFINE_TYPE_WITH_CODE (NMSettingOvsInterface, nm_setting_ovs_interface, NM_TYPE_SETTING,
-                         _nm_register_setting (OVS_INTERFACE, NM_SETTING_PRIORITY_HW_BASE))
-NM_SETTING_REGISTER_TYPE (NM_TYPE_SETTING_OVS_INTERFACE)
+G_DEFINE_TYPE (NMSettingOvsInterface, nm_setting_ovs_interface, NM_TYPE_SETTING)
 
 /*****************************************************************************/
 
@@ -85,11 +83,11 @@ _nm_setting_ovs_interface_verify_interface_type (NMSettingOvsInterface *self,
                                                  gboolean *out_modified,
                                                  GError **error)
 {
-	gboolean has_patch;
 	const char *type;
+	const char *type_from_setting = NULL;
+	const char *type_setting = NULL;
 	const char *connection_type;
 	gboolean is_ovs_connection_type;
-	gboolean missing_patch_setting = FALSE;
 
 	g_return_val_if_fail (NM_IS_SETTING_OVS_INTERFACE (self), FALSE);
 	if (normalize) {
@@ -103,10 +101,7 @@ _nm_setting_ovs_interface_verify_interface_type (NMSettingOvsInterface *self,
 	type = self ? self->type : NULL;
 
 	if (   type
-	    && !NM_IN_STRSET (type,
-	                      "internal",
-	                      "system",
-	                      "patch")) {
+	    && !NM_IN_STRSET (type, "internal", "system", "patch", "dpdk")) {
 		g_set_error (error,
 		             NM_CONNECTION_ERROR,
 		             NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -165,9 +160,26 @@ _nm_setting_ovs_interface_verify_interface_type (NMSettingOvsInterface *self,
 		is_ovs_connection_type = FALSE;
 	}
 
-	has_patch = !!nm_connection_get_setting_by_name (connection, NM_SETTING_OVS_PATCH_SETTING_NAME);
+	if (nm_connection_get_setting_by_name (connection, NM_SETTING_OVS_PATCH_SETTING_NAME)) {
+		type_from_setting = "patch";
+		type_setting = NM_SETTING_OVS_PATCH_SETTING_NAME;
+	}
 
-	if (has_patch) {
+	if (nm_connection_get_setting_by_name (connection, NM_SETTING_OVS_DPDK_SETTING_NAME)) {
+		if (type_from_setting) {
+			g_set_error (error,
+			             NM_CONNECTION_ERROR,
+			             NM_CONNECTION_ERROR_INVALID_PROPERTY,
+			             _("A connection can not have both '%s' and '%s' settings at the same time"),
+			             NM_SETTING_OVS_DPDK_SETTING_NAME,
+			             type_setting);
+			return FALSE;
+		}
+		type_from_setting = "dpdk";
+		type_setting = NM_SETTING_OVS_DPDK_SETTING_NAME;
+	}
+
+	if (type_from_setting) {
 		if (!is_ovs_connection_type) {
 			g_set_error (error,
 			             NM_CONNECTION_ERROR,
@@ -178,20 +190,22 @@ _nm_setting_ovs_interface_verify_interface_type (NMSettingOvsInterface *self,
 			g_prefix_error (error, "%s.%s: ", NM_SETTING_OVS_INTERFACE_SETTING_NAME, NM_SETTING_OVS_INTERFACE_TYPE);
 			return FALSE;
 		}
+
 		if (type) {
-			if (!nm_streq (type, "patch")) {
+			if (!nm_streq (type, type_from_setting)) {
 				g_set_error (error,
 				             NM_CONNECTION_ERROR,
 				             NM_CONNECTION_ERROR_INVALID_PROPERTY,
-				             _("A connection with '%s' setting needs to be of 'patch' interface type, not '%s'"),
-				             NM_SETTING_OVS_PATCH_SETTING_NAME,
+				             _("A connection with '%s' setting needs to be of '%s' interface type, not '%s'"),
+				             type_setting,
+				             type_from_setting,
 				             type);
 				g_prefix_error (error, "%s.%s: ", NM_SETTING_OVS_INTERFACE_SETTING_NAME, NM_SETTING_OVS_INTERFACE_TYPE);
 				return FALSE;
 			}
 			return TRUE;
 		}
-		type = "patch";
+		type = type_from_setting;
 		goto normalize;
 	} else {
 		if (nm_streq0 (type, "patch")) {
@@ -227,8 +241,6 @@ normalize:
 			             _("Missing ovs interface type"));
 			g_prefix_error (error, "%s.%s: ", NM_SETTING_OVS_INTERFACE_SETTING_NAME, NM_SETTING_OVS_INTERFACE_TYPE);
 		}
-		if (missing_patch_setting) {
-		}
 		return NM_SETTING_VERIFY_NORMALIZABLE_ERROR;
 	}
 
@@ -363,29 +375,33 @@ finalize (GObject *object)
 }
 
 static void
-nm_setting_ovs_interface_class_init (NMSettingOvsInterfaceClass *setting_class)
+nm_setting_ovs_interface_class_init (NMSettingOvsInterfaceClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (setting_class);
-	NMSettingClass *parent_class = NM_SETTING_CLASS (setting_class);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	NMSettingClass *setting_class = NM_SETTING_CLASS (klass);
 
-	object_class->set_property = set_property;
 	object_class->get_property = get_property;
-	object_class->finalize = finalize;
-	parent_class->verify = verify;
+	object_class->set_property = set_property;
+	object_class->finalize     = finalize;
+
+	setting_class->verify = verify;
 
 	/**
 	 * NMSettingOvsInterface:type:
 	 *
-	 * The interface type. Either "internal", or empty.
+	 * The interface type. Either "internal", "system", "patch", "dpdk", or empty.
 	 *
 	 * Since: 1.10
 	 **/
-	g_object_class_install_property
-	        (object_class, PROP_TYPE,
-	         g_param_spec_string (NM_SETTING_OVS_INTERFACE_TYPE, "", "",
-	                              NULL,
-	                              G_PARAM_READWRITE |
-	                              G_PARAM_CONSTRUCT |
-	                              NM_SETTING_PARAM_INFERRABLE |
-	                              G_PARAM_STATIC_STRINGS));
+	obj_properties[PROP_TYPE] =
+	    g_param_spec_string (NM_SETTING_OVS_INTERFACE_TYPE, "", "",
+	                         NULL,
+	                         G_PARAM_READWRITE |
+	                         G_PARAM_CONSTRUCT |
+	                         NM_SETTING_PARAM_INFERRABLE |
+	                         G_PARAM_STATIC_STRINGS);
+
+	g_object_class_install_properties (object_class, _PROPERTY_ENUMS_LAST, obj_properties);
+
+	_nm_setting_class_commit (setting_class, NM_META_SETTING_TYPE_OVS_INTERFACE);
 }
