@@ -1,5 +1,3 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-
 /*
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,7 +25,7 @@
 #error "Only <NetworkManager.h> can be included directly."
 #endif
 
-#include <nm-setting.h>
+#include "nm-setting.h"
 
 G_BEGIN_DECLS
 
@@ -61,6 +59,8 @@ G_BEGIN_DECLS
 
 /**
  * NMSettingPpp:
+ *
+ * Point-to-Point Protocol Settings
  */
 struct _NMSettingPpp {
 	NMSetting parent;
