@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /*
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,6 +20,7 @@
 #include "nm-default.h"
 
 #include "nm-dhcp6-config.h"
+
 #include "nm-object-private.h"
 
 G_DEFINE_TYPE (NMDhcp6Config, nm_dhcp6_config, NM_TYPE_DHCP_CONFIG)
@@ -33,7 +33,4 @@ nm_dhcp6_config_init (NMDhcp6Config *config)
 static void
 nm_dhcp6_config_class_init (NMDhcp6ConfigClass *config_class)
 {
-	NMObjectClass *nm_object_class = NM_OBJECT_CLASS (config_class);
-
-	_nm_object_class_add_interface (nm_object_class, NM_DBUS_INTERFACE_DHCP6_CONFIG);
 }
