@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /*
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,9 +21,11 @@
 #ifndef __NM_IP6_CONFIG_H__
 #define __NM_IP6_CONFIG_H__
 
-#include "nm-ip-config.h"
+#if !((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_PRIVATE)
+#error Cannot use this header.
+#endif
 
-G_BEGIN_DECLS
+#include "nm-ip-config.h"
 
 #define NM_TYPE_IP6_CONFIG            (nm_ip6_config_get_type ())
 #define NM_IP6_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_IP6_CONFIG, NMIP6Config))
@@ -42,13 +43,8 @@ typedef struct {
 
 typedef struct {
 	NMIPConfigClass parent;
-
-	/*< private >*/
-	gpointer padding[4];
 } NMIP6ConfigClass;
 
 GType nm_ip6_config_get_type (void);
 
-G_END_DECLS
-
 #endif /* __NM_IP6_CONFIG_H__ */
