@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +19,6 @@
 
 #include "nm-default.h"
 
-#include <string.h>
 #include <sys/types.h>
 #include <signal.h>
 
@@ -111,8 +109,8 @@ done:
 
 static void
 test_secret_agent_cancel_get_secrets (NMSecretAgentOld *agent,
-                                      const gchar      *connection_path,
-                                      const gchar      *setting_name)
+                                      const char       *connection_path,
+                                      const char       *setting_name)
 {
 	g_assert_not_reached ();
 }
@@ -120,7 +118,7 @@ test_secret_agent_cancel_get_secrets (NMSecretAgentOld *agent,
 static void
 test_secret_agent_save_secrets (NMSecretAgentOld                *agent,
                                 NMConnection                    *connection,
-                                const gchar                     *connection_path,
+                                const char                      *connection_path,
                                 NMSecretAgentOldSaveSecretsFunc  callback,
                                 gpointer                         callback_data)
 {
@@ -130,7 +128,7 @@ test_secret_agent_save_secrets (NMSecretAgentOld                *agent,
 static void
 test_secret_agent_delete_secrets (NMSecretAgentOld                  *agent,
                                   NMConnection                      *connection,
-                                  const gchar                       *connection_path,
+                                  const char                        *connection_path,
                                   NMSecretAgentOldDeleteSecretsFunc  callback,
                                   gpointer                           callback_data)
 {
@@ -247,6 +245,9 @@ test_setup (TestSecretAgentData *sadata, gconstpointer test_data)
 	GError *error = NULL;
 
 	sadata->sinfo = nmtstc_service_init ();
+	if (!sadata->sinfo)
+		return;
+
 	sadata->client = nm_client_new (NULL, &error);
 	g_assert_no_error (error);
 
@@ -311,6 +312,9 @@ test_cleanup (TestSecretAgentData *sadata, gconstpointer test_data)
 	GVariant *ret;
 	GError *error = NULL;
 
+	if (!sadata->sinfo)
+		return;
+
 	if (sadata->agent) {
 		if (nm_secret_agent_old_get_registered (sadata->agent)) {
 			nm_secret_agent_old_unregister (sadata->agent, NULL, &error);
@@ -349,10 +353,9 @@ connection_activated_none_cb (GObject *c,
                               gpointer user_data)
 {
 	TestSecretAgentData *sadata = user_data;
-	NMActiveConnection *ac;
 	gs_free_error GError *error = NULL;
 
-	ac = nm_client_activate_connection_finish (sadata->client, result, &error);
+	nm_client_activate_connection_finish (sadata->client, result, &error);
 	g_assert_error (error, NM_AGENT_MANAGER_ERROR, NM_AGENT_MANAGER_ERROR_NO_SECRETS);
 
 	g_main_loop_quit (sadata->loop);
@@ -361,6 +364,9 @@ connection_activated_none_cb (GObject *c,
 static void
 test_secret_agent_none (TestSecretAgentData *sadata, gconstpointer test_data)
 {
+	if (!nmtstc_service_available (sadata->sinfo))
+		return;
+
 	nm_client_activate_connection_async (sadata->client,
 	                                     sadata->connection,
 	                                     sadata->device,
@@ -406,6 +412,9 @@ connection_activated_no_secrets_cb (GObject *c,
 static void
 test_secret_agent_no_secrets (TestSecretAgentData *sadata, gconstpointer test_data)
 {
+	if (!nmtstc_service_available (sadata->sinfo))
+		return;
+
 	g_signal_connect (sadata->agent, "secret-requested",
 	                  G_CALLBACK (secrets_requested_no_secrets_cb),
 	                  sadata);
@@ -457,6 +466,9 @@ secrets_requested_cancel_cb (TestSecretAgent *agent,
 static void
 test_secret_agent_cancel (TestSecretAgentData *sadata, gconstpointer test_data)
 {
+	if (!nmtstc_service_available (sadata->sinfo))
+		return;
+
 	g_signal_connect (sadata->agent, "secret-requested",
 	                  G_CALLBACK (secrets_requested_cancel_cb),
 	                  sadata);
@@ -511,6 +523,9 @@ secrets_requested_good_cb (TestSecretAgent *agent,
 static void
 test_secret_agent_good (TestSecretAgentData *sadata, gconstpointer test_data)
 {
+	if (!nmtstc_service_available (sadata->sinfo))
+		return;
+
 	g_signal_connect (sadata->agent, "secret-requested",
 	                  G_CALLBACK (secrets_requested_good_cb),
 	                  sadata);
@@ -527,7 +542,6 @@ test_secret_agent_good (TestSecretAgentData *sadata, gconstpointer test_data)
 	g_assert_cmpint (sadata->secrets_requested, ==, 1);
 }
 
-
 static void
 async_init_cb (GObject *object, GAsyncResult *result, gpointer user_data)
 {
@@ -567,7 +581,6 @@ test_secret_agent_nm_not_running (void)
 	g_main_loop_unref (loop);
 }
 
-
 static void
 registered_changed (GObject *object, GParamSpec *pspec, gpointer user_data)
 {
@@ -585,6 +598,9 @@ test_secret_agent_auto_register (void)
 	GError *error = NULL;
 
 	sinfo = nmtstc_service_init ();
+	if (!nmtstc_service_available (sinfo))
+		return;
+
 	loop = g_main_loop_new (NULL, FALSE);
 
 	agent = test_secret_agent_new ();
@@ -612,6 +628,8 @@ test_secret_agent_auto_register (void)
 
 	/* Restart test service */
 	sinfo = nmtstc_service_init ();
+	g_assert (nmtstc_service_available (sinfo));
+
 	g_main_loop_run (loop);
 	g_assert (nm_secret_agent_old_get_registered (agent));
 
