@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* NetworkManager -- Network link manager
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,8 +22,6 @@
 
 #include "nm-device-ethernet.h"
 
-G_BEGIN_DECLS
-
 #define NM_TYPE_DEVICE_VETH            (nm_device_veth_get_type ())
 #define NM_DEVICE_VETH(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_DEVICE_VETH, NMDeviceVeth))
 #define NM_DEVICE_VETH_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  NM_TYPE_DEVICE_VETH, NMDeviceVethClass))
@@ -34,11 +31,9 @@ G_BEGIN_DECLS
 
 #define NM_DEVICE_VETH_PEER "peer"
 
-typedef NMDeviceEthernet NMDeviceVeth;
-typedef NMDeviceEthernetClass NMDeviceVethClass;
+typedef struct _NMDeviceVeth NMDeviceVeth;
+typedef struct _NMDeviceVethClass NMDeviceVethClass;
 
 GType nm_device_veth_get_type (void);
 
-G_END_DECLS
-
-#endif	/* NM_DEVICE_VETH_H */
+#endif /* __NETWORKMANAGER_DEVICE_VETH_H__ */
